@@ -30,9 +30,9 @@
 #ifndef MOZC_BASE_UTIL_H_
 #define MOZC_BASE_UTIL_H_
 
-#include <vector>
 #include <string>
 #include <utility>
+#include <vector>
 
 #include "base/base.h"
 
@@ -105,6 +105,12 @@ class Util {
     return result;
   }
 
+  // Determines whether the beginning of |str| matches |prefix|.
+  static bool StartsWith(const string &str, const string &prefix);
+
+  // Determines whether the end of |str| matches |suffix|.
+  static bool EndsWith(const string &str, const string &suffix);
+
   // Strip a heading UTF-8 BOM (binary order mark) sequence (= \xef\xbb\xbf).
   static void StripUTF8BOM(string *line);
 
@@ -122,6 +128,20 @@ class Util {
   static bool SafeStrToUInt32(const string &str, uint32 *value);
   static bool SafeStrToUInt64(const string &str, uint64 *value);
 
+  // Convert the string to a double.  Returns true if success or false if the
+  // string is in the wrong format.
+  // If |str| is a hexadecimal number like "0x1234", the result depends on
+  // compiler.  It returns false when compiled by VisualC++.  On the other hand
+  // it returns true and sets correct value when compiled by gcc.
+  static bool SafeStrToDouble(const string &str, double *value);
+
+#ifndef SWIG
+  // C++ string version of sprintf.
+  static string StringPrintf(const char* format, ...)
+      // Tell the compiler to do printf format string checking.
+      PRINTF_ATTRIBUTE(1, 2);
+#endif
+
   // Chop the return characters (i.e. '\n' and '\r') at the end of the
   // given line.
   static bool ChopReturns(string *line);
@@ -141,6 +161,7 @@ class Util {
   static uint32 Fingerprint32WithSeed(const char *str,
                                       uint32 seed);
 #endif
+  static uint32 Fingerprint32WithSeed(uint32 num, uint32 seed);
 
   // 64bit Fingerprint
   static uint64 Fingerprint(const string &key);
@@ -236,6 +257,9 @@ class Util {
   // half-width-katakana area.
   static bool IsHalfWidthKatakanaSymbol(const string &input);
 
+  // return true if one or more Kana-symbol characters are
+  // in the input.
+  static bool IsKanaSymbolContained(const string &input);
 
   static void NormalizeVoicedSoundMark(const string &input,
                                        string *output);
@@ -304,8 +328,11 @@ class Util {
   // does nothing in other platforms.
   static string NormalizeDirectorySeparator(const string &path);
 
-  // return ~/.mozc for Unix/Mac
-  // return %APPDATA%\\google\\mozc for Windows 2000, XP and Vista
+  // return "~/.mozc" for Unix/Mac
+  // return "%USERPROFILE%\\Local Settings\\Application\\"
+  //        "Google\\Google Japanese Input" for Windows XP.
+  // return "%USERPROFILE%\\AppData\\LocalLow\\"
+  //        "Google\\Google Japanese Input" for Windows Vista and later.
   static string GetUserProfileDirectory();
 
   // return ~/Library/Logs/Mozc for Mac
@@ -320,6 +347,27 @@ class Util {
   // build but available from those tests.
   static void SetUserProfileDirectory(const string &path);
 
+#ifdef OS_WINDOWS
+  // From an early stage of the development of Mozc, we have somehow abused
+  // CHECK macro assuming that any failure of fundamental APIs like
+  // ::SHGetFolderPathW or ::SHGetKnownFolderPathis is worth being notified
+  // as a crash.  But the circumstances have been changed.  As filed as
+  // b/3216603, increasing number of instances of various applications begin
+  // to use their own sandbox technology, where these kind of fundamental APIs
+  // are far more likely to fail with an unexpected error code.
+  // EnsureVitalImmutableDataIsAvailable is a simple fail-fast mechanism to
+  // this situation.  This function simply returns false instead of making
+  // the process crash if any of following functions cannot work as expected.
+  // - IsVistaOrLaterCache
+  // - SystemDirectoryCache
+  // - ProgramFilesX86Cache
+  // - LocalAppDataDirectoryCache
+  // TODO(taku,yukawa): Implement more robust and reliable mechanism against
+  //   sandboxed environment, where such kind of fundamental APIs are far more
+  //   likely to fail.  See b/3216603.
+  static bool EnsureVitalImmutableDataIsAvailable();
+#endif  // OS_WINDOWS
+
   // return the directory name where the mozc server exist.
   static string GetServerDirectory();
 
@@ -401,6 +449,10 @@ class Util {
   // If str has mixed scripts, this function returns UNKNOWN_SCRIPT
   static ScriptType GetScriptType(const string &str);
 
+  // The same as GetScryptType(), but it ignores symbols
+  // in the |str|.
+  static ScriptType GetScriptTypeWithoutSymbols(const string &str);
+
   // return true if all script_type in str is "type"
   static bool IsScriptType(const string &str, ScriptType type);
 
@@ -453,6 +505,15 @@ class Util {
   // return true if the version of Windows is x64 Edition.
   static bool IsWindowsX64();
 
+  enum IsWindowsX64Mode {
+    IS_WINDOWS_X64_DEFAULT_MODE,
+    IS_WINDOWS_X64_EMULATE_32BIT_MACHINE,
+    IS_WINDOWS_X64_EMULATE_64BIT_MACHINE,
+  };
+
+  // For unit tests, this function overrides the behavior of |IsWindowsX64|.
+  static void SetIsWindowsX64ModeForTest(IsWindowsX64Mode mode);
+
   // return system directory. If failed, return NULL.
   // You need not to delete the returned pointer.
   // This function is thread safe.
